#include  <sys/types.h>
#include <regex.h>
#include <stdio.h>


main()
{
  char *p, *p2;
  char  buf[200];
  snprintf(buf, 200,"\rabcsntaoehsnatou\r\n\rasoeusontehu\r\nasoeusontehu\r\nasoeusontehu");

    for (p = buf, p2 = buf ; *p!='\0'; p++) 
	  { 
		*p2 = *p; 
		if (*p != '\r') p2++; 
	  }

	printf(buf);
}
