#ifndef __ICQ_H
#define __ICQ_H

typedef struct _ICQ_SERVER_CONNECT_REC ICQ_SERVER_CONNECT_REC;
typedef struct _ICQ_SERVER_REC ICQ_SERVER_REC;
typedef struct _ICQ_CHANNEL_REC ICQ_CHANNEL_REC;

#define ICQ_PROTOCOL (chat_protocol_lookup("ICQ"))

#endif
