/*
 icq-session.c : irssi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "module.h"
#include "signals.h"

#include "icq-servers.h"

static void sig_connected(ICQ_SERVER_REC * server)
{
	if (!IS_ICQ_SERVER(server) || !server->session_reconnect)
		return;

	server->connected = TRUE;
	signal_emit("event connected", 1, server);
}

void icq_session_init(void)
{
	signal_add("server connected", (SIGNAL_FUNC) sig_connected);
}

void icq_session_deinit(void)
{
	signal_remove("server connected", (SIGNAL_FUNC) sig_connected);
}
